from ply import lex

tokens = [
    "ID",
    "STR",
    "MLSTR",
    "DATE",
    "TIME",
    "DATETIME",
    "INT",
    "HEX",
    "BIN",
    "OCT",
    "FLOAT",  # need to implement exponents check https://toml.io/en/
    "INF",
    "NAN",
    "COMMENT",
]


# needs to check if datetime is valid
def t_DATETIME(t):
    r"\d{4}-\d{2}-\d{2}[T ]\d{2}:\d{2}:\d{2}(\.\d{1,6})?(Z|[+-]\d{2}:\d{2})"
    return t


# needs to check if date is valid
def t_DATE(t):
    r"\d{4}-\d{2}-\d{2}"
    return t


# needs to check if time is valid
def t_TIME(t):
    r"\d{2}:\d{2}:\d{2}(\.\d{1,6})?"
    return t


# needs number grouping (example : flt8 = 224_617.445_991_228)
def t_FLOAT(t):
    r"[+-]?\d+(\s*\.\s*\d+)?([eE][-+]?\d+)?"
    #case where float appears on the left side with spaces in between
    if t.value.__contains__(' '):
        t.type = "ID"
        t.value = [s.strip(' ')  for s in t.value.split('.')]
    return t


# needs number grouping (example : int6 = 5_349_221)
def t_INT(t):
    r"[-+]?\d+"
    return t


# needs number grouping (example : hex3 = 0xdead_beef)
def t_HEX(t):
    r"0x[0-9a-fA-F]+"
    return t


def t_BIN(t):
    r"0b[01]+"
    return t


def t_OCT(t):
    r"0o[0-7]+"
    return t


def t_INF(t):
    r"[-+]?inf"
    return t


def t_NAN(t):
    r"[+-]?nan"
    return t


# booleans are always lowercase
def t_BOOL(t):
    r"(true|false)"
    return t


# ID needs to be the last so it doesnt catch everything (literally)
def t_ID(t):
    r"(([\w_]+)|(\"[\w_]+\"|\'[\w_]+\')\s*\.\s*([\w_]+|\"[\w_]+\"|\'[\w_]+\'))(\s*\.\s*([\w_]+|\"[\w_]+\"|\'[\w_]+\'))*"
    t.value = [s.strip(" \"'") for s in t.value.split('.')]
    return t


def t_MLSTR(t):
    r"(\"\"\"(?:[^\"\\]|\\.)+\"\"\")|(\'\'\'(?:[^\'\\]|\\.)+\'\'\')"
    return t


# STR needs to be the first one to catch
def t_STR(t):
    r"(\"(?:[^\"\\]|\\.)+\")|(\'[^\']+\')"
    t.value = t.value.strip("\"'")
    return t


def t_COMMENT(t):
    r"\#.*"
    pass


def t_error(t):
    print("Illegal character '%s'" % t.value[0])
    t.lexer.skip(1)


t_ignore = "\n\t "

literals = "[]{},="

lexer = lex.lex()
